!---------------------------------------------------------------------------------------
! FLEXINVERT : readinput
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Reads input data from inversion 
!---------------------------------------------------------------------------------------

subroutine readinput()

  use mod_var

  implicit none
!  character(len=100)             :: pn
!  pn= '/jetfs/spack/opt/spack/linux-rhel8-skylake/_avx512/gcc-8.3.1'//&
!  &'/netcdf-fortran-4.5.3-kfd2vkj7m6cremij4nunxwqfcrqjovcf/netcdf.inc'
  include 'netcdf.inc'
!  include pn 
  integer :: i,j,n,ix,jy,nid,ierr,xid,yid,zid,nx,ny,nz,varid
  real, dimension(:,:), allocatable :: tmp
  real :: area_grid

  !---------------------------------------------------------------
  ! prior flux at fine resolution
  !---------------------------------------------------------------

  ! open netcdf file
  write(logid,*) 'Reading analysis.nc'
  ierr=nf_open(trim(dirinvert)//'/'//'analysis.nc',NF_NOWRITE,nid)
  if(ierr.ne.0) then
    write(logid,*) 'Error opening analysis.nc'
  endif

  ! inquire about variables
  ierr=nf_inq_dimid(nid,'longitude',xid)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading dimension: longitude'
  endif
  ierr=nf_inq_dimid(nid,'latitude',yid)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading dimension: latitude'
  endif  
  ierr=nf_inq_dimid(nid,'time',zid)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading dimension: time'
  endif

  ! read dimension sizes
  ierr=nf_inq_dimlen(nid,xid,nlon)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading dimension length'
  endif
  ierr=nf_inq_dimlen(nid,yid,nlat)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading dimension length'
  endif
  ierr=nf_inq_dimlen(nid,zid,ntime)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading dimension length'
  endif

  allocate(lon(nlon))
  allocate(lat(nlat))
  allocate(time(ntime))
  allocate(emisprior(nlon,nlat,ntime))
  allocate(emispost_ana(nlon,nlat,ntime))

  ! read variables
  ierr=nf_inq_varid(nid,'longitude',varid)
  ierr=nf_get_var_real(nid,varid,lon)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading longitude'
  endif
  ierr=nf_inq_varid(nid,'latitude',varid)
  ierr=nf_get_var_real(nid,varid,lat)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading latitude'
  endif
  ierr=nf_inq_varid(nid,'time',varid)
  ierr=nf_get_var_real(nid,varid,time)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading time'
  endif
  ierr=nf_inq_varid(nid,'fpri',varid)
  ierr=nf_get_var_real(nid,varid,emisprior)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading emisprior'
  endif
  ierr=nf_inq_varid(nid,'fpos',varid)
  ierr=nf_get_var_real(nid,varid,emispost_ana)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading emispost_ana'
  endif

  ierr=nf_close(nid)

  ! multiply by numscale and outheight
  emisprior=emisprior*numscale/outheight
  emispost_ana=emispost_ana*numscale/outheight

  !CGZ Adjust nemsteps to number of timesteps, no need to set in SETTINGS_constraint
  nemsteps=ntime

  !---------------------------------------------------------------
  ! grid definition
  !---------------------------------------------------------------

  ! open netcdf file
  nx=nlon
  nxgrid=nx
  ny=nlat
  nygrid=ny
  write(logid,*) 'Reading nbox_xy.txt'
  !print*, 'nlon:', nlon, ' nx:', nx,'nlat:', nlat, ' ny:', ny
  allocate(nbox_xy(nx,ny))
  allocate(tmp(nx,ny))

  open(100,file=trim(dirinvert)//'/nbox_xy.txt',action='read',status='old',iostat=ierr)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading nbox_xy.txt'
  endif
  do i=1,nx
    read(100,*) tmp(i,:)
    !write(logid,*) , tmp(i,:)
  end do
  nbox_xy=int(tmp)
  nbox=maxval(nbox_xy)

  !print*, nbox_xy
  close(100)
  deallocate(tmp)

  !---------------------------------------------------------------
  ! grid box area
  !---------------------------------------------------------------

  allocate(area_box(nbox))

  open(100,file=trim(dirinvert)//'/area_box.txt',action='read',status='old',iostat=ierr)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading area_box.txt'
  endif
  do i=1,nbox
    read(100,*) area_box(i)
  end do
  !print*, area_box
  close(100)


  !---------------------------------------------------------------
  ! posterior error covariance
  !---------------------------------------------------------------
  write(logid,*) 'Reading cova.txt or izwork.txt'
  nvar=nbox*nemsteps

  allocate(cova(nvar,nvar))
  cova(:,:)=0
  open(100,file=trim(dirinvert)//'/izwork.txt',action='read',status='old',iostat=ierr)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading izwork.txt, reading cova.txt'
    !CGZ: try reading cova.txt
    open(100,file=trim(dirinvert)//'/cova.txt',action='read',status='old',iostat=ierr)
    if(ierr.ne.0) then
      write(logid,*) 'Error reading cova.txt'
    endif
  endif
  do i=1,nvar
    read(100,*) cova(i,:)
   ! print*, cova(i,:)
  end do
  close(100)


  !---------------------------------------------------------------
  ! prior state vector
  !---------------------------------------------------------------

  allocate(xb(nvar))
  
  write(logid,*) 'Calculating prior from analysis.nc'
  xb(:) = 0.
  do n = 1, ntime
    do ix = 1, nlon
      do jy = 1, nlat
        call gridarea(lat(1)+(float(jy)-0.5)*dy,dy,dx,area_grid)
        if ( nbox_xy(ix,jy).gt.0 ) then ! only boxes over land
          xb((n-1)*nbox+nbox_xy(ix,jy)) = xb((n-1)*nbox+nbox_xy(ix,jy)) + &
                                          emisprior(ix,jy,n) * area_grid
        endif
        if (ix.eq.1) then
        endif
      end do
    end do
  end do
  do n = 1,nemsteps
    xb((n-1)*nbox+1:(n)*nbox)=xb((n-1)*nbox+1:(n)*nbox)/area_box(:)
  end do

  !---------------------------------------------------------------
  ! posterior state vector
  !---------------------------------------------------------------

  allocate(x0(nvar))
  
  !write(logid,*) 'Calculating posterior from analysis.nc'
  !x0(:) = 0.
  !do n = 1, ntime
  !  do ix = 1, nlon
  !    do jy = 1, nlat
  !      call gridarea(lat(1)+(float(jy)-0.5)*dy,dy,dx,area_grid)
  !      if ( nbox_xy(ix,jy).gt.0 ) then ! only boxes over land
  !        x0((n-1)*nbox+nbox_xy(ix,jy)) = xb((n-1)*nbox+nbox_xy(ix,jy)) + &
  !                                        emispost_ana(ix,jy,n) * area_grid
  !      endif
  !      if (ix.eq.1) then
  !      endif
  !    end do
  !  end do
  !end do
  !do n = 1,nemsteps
  !  x0((n-1)*nbox+1:(n)*nbox)=x0((n-1)*nbox+1:(n)*nbox)/area_box(:)
  !end do

  write(logid,*) 'Reading posterior.txt'
  open(100,file=trim(dirinvert)//'posterior.txt',action='read',status='old',iostat=ierr)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading posterior.txt'
  endif
  write(logid,*) 'nvar:', nvar
  !write(logid,*) 'nemsteps:', nemsteps
  write(logid,*) 'nbox:', nbox
  do i=1,nvar
    read(100,*) x0(i)
  end do
  close(100)
  ! multiply by numscale and outheight and add prior since flexinvert optimises offstets
  x0=x0*numscale/outheight+xb

  !---------------------------------------------------------------
  ! total global constraint with error
  !---------------------------------------------------------------

  if (contype.eq.1) then
    allocate(ineqvec(nemsteps))
    allocate(tgerrvec(nemsteps))
    write(logid,*) 'Reading ',trim(tgconpath)
    open(100,file=trim(tgconpath),action='read',status='old',iostat=ierr)
    if (ierr.ne.0) then
      write(logid,*) 'Error reading ',trim(tgconpath)
    endif

    read(100,*)
    do i=1,nemsteps
      ! read total global flux and associated error in unit kg/s*numscale/outheight
      read(100,*) ineqvec(i), tgerrvec(i)
      ineqvec(i) = ineqvec(i)*numscale/outheight
      tgerrvec(i) = (tgerrvec(i)*numscale)**2
    end do
    close(100)
  endif

end subroutine readinput


