!---------------------------------------------------------------------------------------
! GRID_TO_NCDF: init_convert
!---------------------------------------------------------------------------------------
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!
!  Copyright 2017, Rona Thompson
!---------------------------------------------------------------------------------------
!
!> init_convert
!! 
!! Purpose:    
!!
!---------------------------------------------------------------------------------------

subroutine init_convert(files, config, nr, timestamp)

  use mod_var
  use mod_settings
  use mod_flexpart
  use mod_dates

  implicit none

  type (files_t),  intent(in out)        :: files
  type (config_t), intent(in)            :: config
  integer,         intent(in)            :: nr
  integer,         intent(in)            :: timestamp

  character(max_path_len)                                :: filename, filedates, path_flexpart, path_output
  character(len=max_path_len), dimension(:), allocatable :: filelist
  character(len=8)                                       :: adate
  character(len=6)                                       :: atime
  character(len=3)                                       :: anspec
  logical                                                :: lexist
  integer                                                :: nfiles, i, ierr, n
  integer                                                :: jjjjmmdd, hhmiss 
  real, dimension(:,:,:), allocatable                    :: grid
  real(kind=8), dimension(:), allocatable                :: levels
  real(kind=8)                                           :: jdate
  integer                                                :: numpoint
  integer                                                :: ibdate, ibtime
  integer, dimension(maxpoint)                           :: releases
  real                                                   :: bndx, bndy, delx, dely
  integer                                                :: numx, numy, numz

  ! list all grid files in directory
  if ( trim(config%filefreq).eq.'month' ) then
    write(adate,'(I6.6)') timestamp/100
    print*, adate
  else if ( trim(config%filefreq).eq.'day' ) then
    write(adate,'(I8.8)') timestamp
  endif
  path_flexpart = trim(files%path_flexpart)//trim(recname(nr))//'/'//trim(adate)//'/'
  path_output = trim(files%path_output)//trim(recname(nr))//'/'
  print*, path_flexpart
  print*, path_output
  call system('mkdir '//trim(path_output))
  call system('ls '//trim(path_flexpart)//' | grep grid_initial_ | wc -l > '//trim(path_output)//'files.txt')
  call system('ls '//trim(path_flexpart)//' | grep grid_initial_  >> '//trim(path_output)//'files.txt')

  ! read file list
  open(100,file=trim(path_output)//'files.txt',status="old",action="read",iostat=ierr)
  if ( ierr.ne.0 ) then
    print*, 'ERROR: problem read files.txt'
    stop
  endif
  read(100,*) nfiles
  allocate( filelist(nfiles) ) 
  do i = 1, nfiles
    read(100,fmt='(A)',iostat=ierr) filelist(i)
    if (ierr.ne.0) exit
  end do
  close(100)

  ! read header for this month to get ngrid
  filename = trim(path_flexpart)//'header'
  inquire(file=trim(filename),exist=lexist)
  if ( .not.lexist ) then
    print*, 'ERROR: cannot find flexpart header'
    stop
  endif
  print*, 'Reading flexpart header: '//trim(filename)
  call read_header(filename, numpoint, ibdate, ibtime, releases, &
                   numx, numy, numz, bndx, bndy, delx, dely)
  print*, 'numx, numy, numz, bndx, bndy, delx, dely = ',numx,numy,numz,bndx,bndy,delx,dely
  allocate( grid(nxgrid,nygrid,nzgrid) )
  allocate( levels(nzgrid) )
  ! trick to use write_ncdf also for grid_initial
  ngrid = nzgrid
  levels(:) = dble(outheight(1:nzgrid))

  ! loop over files
  filedates = trim(path_flexpart)//'dates'
  do i = 1, nfiles
    ! read binary grid files
    adate = filelist(i)(14:21)
    if ( trim(config%filefreq).eq.'month' ) then
      atime = filelist(i)(22:27)
    else if ( trim(config%filefreq).eq.'day' ) then
      ! assume satellite -> get number of retrieval
      atime = filelist(i)(23:28)
    endif 
    print*, 'adate = ',adate
    print*, 'atime = ',atime
    read(adate,*) jjjjmmdd
    read(atime,*) hhmiss
    print*, 'jjjjmmdd = ',jjjjmmdd
    print*, 'hhmiss = ',hhmiss
    if ( config%linversionout ) then
      jdate = juldate(jjjjmmdd, hhmiss)
    else
      jdate = juldate(datei,timei)
    endif
!    print*, 'jdate = ',jdate
    filename = trim(path_flexpart)//trim(filelist(i))
    if ( config%linversionout ) then
      call read_init(filename, grid)
    else
      call read_init_std(filename, grid)
    endif
    ! write to ncdf
    if ( trim(config%filefreq).eq.'month' ) then
      filename = trim(path_output)//'grid_initial_'//adate//atime//'_001.nc'
    else if ( trim(config%filefreq).eq.'day' ) then
      ! assume satellite
      filename = trim(path_output)//'grid_initial_'//adate//'_'//atime//'.nc'
    endif
    call write_ncdf(config, filename, levels, grid)
  end do  

  deallocate( grid )
  deallocate( levels )

end subroutine init_convert

